@@ -324,6 +324,77 @@ static void point_proj_double(point_t R, element_t Rz, point_t P, element_t z)
     element_clear(e3);
 }
 
+static void point_proj_mixin(point_t R, element_t Rz, point_t P, element_t Pz, point_t Q)
+{
+    element_t e2, e3, e5, e6, e9;
+    element_init(e2, Rz->field);
+    element_init(e3, Rz->field);
+    element_init(e5, Rz->field);
+    element_init(e6, Rz->field);
+    element_init(e9, Rz->field);
+    element_ptr Px = P->x;
+    element_ptr Py = P->y;
+    element_ptr Qx = Q->x;
+    element_ptr Qy = Q->y;
+
+    //e2 = Qx Pz^2, e5 holds Pz^2 until later
+    element_square(e5, Pz);
+    element_mul(e2, e5, Qx);
+
+    //e3 = Px - e2
+    element_sub(e3, Px, e2);
+
+    //e5 = Qy Pz^3 TODO: check this one
+    element_mul(e5, e5, Qy);
+    element_mul(e5, e5, Pz);
+
+    //e6 = Py - e5
+    element_sub(e6, Py, e5);
+
+    //e7 = Px + e2, use e2 to double for e7
+    element_add(e2, Px, e2);
+
+    //e8 = Py + e5, use e5 to double for e8
+    element_add(e5, Py, e5);
+
+    //Rz = Pz e3
+    element_mul(Rz, Pz, e3);
+
+    //Rx = e6^2 - e7 e3^2
+    //e3 now holds e3^3,
+    //e9 holds e7 e3^2
+    element_square(e9, e3);
+    element_mul(e3, e9, e3);
+    element_square(R->x, e6);
+    element_mul(e9, e2, e9);
+    element_sub(R->x, R->x, e9);
+
+    //e9 = e7 e3^2 - 2 Rx
+    element_sub(e9, e9, R->x);
+    element_sub(e9, e9, R->x);
+
+    //Ry = (e9 e6 - e8 e3^3)/2
+    element_mul(e9, e9, e6);
+    element_mul(e5, e5, e3);
+    element_sub(e9, e9, e5);
+    //TODO: rewrite so it doesn't access internal structure
+    {
+	mpz_ptr z0 = e9->data;
+	if (mpz_odd_p(z0)) {
+	    mpz_add(z0, z0, e9->field->order);
+	}
+	mpz_tdiv_q_2exp(R->y->data, z0, 1);
+    }
+
+    R->inf_flag = 0;
+
+    element_clear(e2);
+    element_clear(e3);
+    element_clear(e5);
+    element_clear(e6);
+    element_clear(e9);
+}
+
 //convert P from weighted projective (Jacobian) to affine
 //i.e. (X, Y, Z) --> (X/Z^2, Y/Z^3)
 //also sets z to 1
@@ -465,6 +536,166 @@ static void pairing_init_c_param_odd_k(pairing_t pairing, c_param_t param)
     element_clear(b);
 }
 
+static void cc_miller_no_denom_proj(element_t res, mpz_t q, point_t P,
+	element_ptr Qx, element_ptr Qy, fieldmap mapbase)
+{
+    //collate divisions
+    int m;
+    element_t v;
+    point_t Z;
+    element_t a, b, c;
+    common_curve_ptr cc = P->curve->data;
+    element_t t0, t1;
+    element_t e0, e1;
+    element_t z;
+
+    void do_vertical(void)
+    {
+	if (point_is_inf(Z)) {
+	    return;
+	}
+	mapbase(e0, Z->x);
+	element_sub(e0, Qx, e0);
+	element_mul(v, v, e0);
+    }
+
+    void do_tangent(void)
+    {
+	element_ptr Zx = Z->x;
+	element_ptr Zy = Z->y;
+
+	if (point_is_inf(Z)) {
+	    return;
+	}
+
+	if (element_is0(Zy)) {
+	    //order 2
+	    do_vertical();
+	    return;
+	}
+	//a = -(3x^2 + cca z^4)
+	//b = 2 y z^3
+	//c = -(2 y^2 + x a)
+	//a = z^2 a
+	element_square(t0, z);
+	element_square(a, t0);
+	element_mul(a, a, cc->a);
+	element_square(b, Zx);
+	element_mul_si(b, b, 3);
+	element_add(a, a, b);
+	element_neg(a, a);
+
+	element_mul(b, z, t0);
+	element_mul(b, b, Zy);
+	element_mul_si(b, b, 2);
+
+	element_mul(c, Zx, a);
+	element_mul(a, a, t0); //frees e0, don't need z^2 anymore
+	element_square(t0, Zy);
+	element_mul_si(t0, t0, 2);
+	element_add(c, c, t0);
+	element_neg(c, c);
+
+	//TODO: use poly_mul_constant?
+	mapbase(e0, a);
+	element_mul(e0, e0, Qx);
+	mapbase(e1, b);
+	element_mul(e1, e1, Qy);
+	element_add(e0, e0, e1);
+	mapbase(e1, c);
+	element_add(e0, e0, e1);
+	element_mul(v, v, e0);
+    }
+
+    void do_line(void)
+    {
+	element_ptr Ax = Z->x;
+	element_ptr Ay = Z->y;
+	element_ptr Bx = P->x;
+	element_ptr By = P->y;
+
+	//we assume B is never O
+	if (point_is_inf(Z)) {
+	    do_vertical();
+	    return;
+	}
+
+	if (!element_cmp(Ax, Bx)) {
+	    if (!element_cmp(Ay, By)) {
+		do_tangent();
+		return;
+	    }
+	    do_vertical();
+	    return;
+	}
+
+	element_mul(t0, Ax, z);
+	element_mul(t1, z, z);
+	element_mul(t1, t1, z);
+
+	element_mul(a, By, t1);
+	element_sub(a, Ay, a);
+
+	element_mul(b, Bx, t1);
+	element_sub(b, b, t0);
+
+	element_mul(t0, t0, By);
+	element_mul(c, Ay, Bx);
+	element_sub(c, t0, c);
+
+	mapbase(e0, a);
+	element_mul(e0, e0, Qx);
+	mapbase(e1, b);
+	element_mul(e1, e1, Qy);
+	element_add(e0, e0, e1);
+	mapbase(e1, c);
+	element_add(e0, e0, e1);
+	element_mul(v, v, e0);
+    }
+
+    element_init(a, P->curve->field);
+    element_init(b, P->curve->field);
+    element_init(c, P->curve->field);
+    element_init(t0, P->curve->field);
+    element_init(t1, P->curve->field);
+    element_init(e0, res->field);
+    element_init(e1, res->field);
+    element_init(z, a->field);
+    element_set1(z);
+
+    element_init(v, res->field);
+    point_init(Z, P->curve);
+
+    point_set(Z, P);
+
+    element_set1(v);
+    m = mpz_sizeinbase(q, 2) - 2;
+
+    while(m >= 0) {
+	element_square(v, v);
+	do_tangent();
+	point_proj_double(Z, z, Z, z);
+	if (mpz_tstbit(q, m)) {
+	    do_line();
+	    point_proj_mixin(Z, z, Z, z, P);
+	}
+	m--;
+    }
+
+    element_set(res, v);
+
+    element_clear(v);
+    point_clear(Z);
+    element_clear(a);
+    element_clear(b);
+    element_clear(c);
+    element_clear(t0);
+    element_clear(t1);
+    element_clear(e0);
+    element_clear(e1);
+    element_clear(z);
+}
+
 static void cc_miller_no_denom(element_t res, mpz_t q, point_t P,
 	element_ptr Qx, element_ptr Qy, fieldmap mapbase)
 {
