@@ -246,11 +246,6 @@ static void e_miller(element_t res, point_t P,
 
     void do_line(element_ptr e, element_ptr edenom, point_ptr A, point_ptr B)
     {
-	//a = -(B.y - A.y) / (B.x - A.x);
-	//b = 1;
-	//c = -(A.y + a * A.x);
-	//but we'll multiply by B.x - A.x to avoid division
-
 	element_ptr Ax = A->x;
 	element_ptr Ay = A->y;
 	element_ptr Bx = B->x;
@@ -273,10 +268,9 @@ static void e_miller(element_t res, point_t P,
 
 	element_sub(b, Bx, Ax);
 	element_sub(a, Ay, By);
-	element_mul(t0, b, Ay);
-	element_mul(c, a, Ax);
-	element_add(c, c, t0);
-	element_neg(c, c);
+	element_mul(c, Ax, By);
+	element_mul(t0, Ay, Bx);
+	element_sub(c, c, t0);
 
 	element_mul(e0, a, numx);
 	element_mul(e1, b, numy);
@@ -433,11 +427,6 @@ static void cc_miller(element_t res, mpz_t q, point_t P,
 
     void do_line(element_ptr e)
     {
-	//a = -(B.y - A.y) / (B.x - A.x);
-	//b = 1;
-	//c = -(A.y + a * A.x);
-	//but we'll multiply by B.x - A.x to avoid division
-
 	element_ptr Ax = Z->x;
 	element_ptr Ay = Z->y;
 	element_ptr Bx = P->x;
@@ -460,10 +449,9 @@ static void cc_miller(element_t res, mpz_t q, point_t P,
 
 	element_sub(b, Bx, Ax);
 	element_sub(a, Ay, By);
-	element_mul(t0, b, Ay);
-	element_mul(c, a, Ax);
-	element_add(c, c, t0);
-	element_neg(c, c);
+	element_mul(c, Ax, By);
+	element_mul(t0, Ay, Bx);
+	element_sub(c, c, t0);
 
 	element_mul(e0, a, Qx);
 	element_mul(e1, b, Qy);
