@@ -3,19 +3,15 @@
 
 int main(void)
 {
-    cc_param_t cc_param;
     pairing_t pairing;
     bb_sys_param_t param;
     bb_public_key_t pk;
     bb_private_key_t sk;
     unsigned char *sig;
 
-    cc_param_init(cc_param);
-    printf("reading curve parameters from stdin...\n");
-    cc_param_inp_str(cc_param, stdin);
-    printf("initializing pairing...\n");
+    printf("initializing pairing from stdin...\n");
 
-    pairing_init_cc_param(pairing, cc_param);
+    pairing_init_inp_str(pairing, stdin);
     printf("generating BLS system parameters...\n");
     bb_gen_sys_param(param, pairing);
     printf("generating key pair...\n");
